/**
 * Clase que permite la creacion de un robot con sensores.
 * Implementa un robot movible con un vector velocidad, posicion, sensores de
 * distancia, un mundo en el que se situa y una serie de variables internas
 * necesarias para su funcionamiento.
 *
 * Autor: Matias Contreras, Francisco Frez, Damian Quiroz.
 * Version: 2.0
 * Since: 2018-07-24
*/

#ifndef ROBOT_h				// Comprueba si la etiqueta no esta definida
#define ROBOT_h				// Crea una bandera en tiempo de compilacion

#include "Vector2D.h"		// Biblioteca de cabecera que permite usar vectores

class Robot{
	public:
	/* Constructores y metodos */
		Robot(): _position(Vector2D(0,0)), _velocity(Vector2D(0,0)) {}
		Robot(Vector2D position, Vector2D velocity): _position(position), _velocity(velocity) {}
		void turnLeft();
		void turnRight();
		void moveDelta_t(double delta_t);
		void operator=(Robot robot);
		string getDescription() {return _position.getDescription();}
		string toString() {return _position.toString();}
		Vector2D getPosition() {return _position;}
		Vector2D getVelocity() {return _velocity;}

	private:
	/* Definicion de variables */
		Vector2D _position;
		Vector2D _velocity;
};

#endif		/* Codigo se incluye en el paquete
 	 	 	 * que se envia al compilador hasta aqui */
