/**
 * Implementa los constructores y metodos esenciales que le dan vida al robot
 * (posicion, velocidad, giros en alguna direccion, entre otros).
 *
 * Autor: Matias Contreras, Francisco Frez, Damian Quiroz.
 * Version: 1.0
 * Since: 2018-07-21
*/

#include "Robot.h"		// Biblioteca de cabecera que define los constructores y metodos
#include "MyWorld.h"	// Biblioteca de cabecera que permite el uso del mundo simulado
using namespace std;	// Incluye espacio de nombres para palabras reservadas

Robot::Robot() {
	_position = Vector2D(0,0);
	_velocity = Vector2D(0,0);
	_world = NULL;
	_rightSensor = NULL;
	_frontSensor = NULL;
	_leftSensor = NULL;
	/* Constructor que inicializa las variables a utilizar */
}

Robot::Robot(Vector2D position, Vector2D velocity, double sensorRange, MyWorld* world) {
	Vector2D direction;
	_position = position;
	_velocity = velocity;
	_world = world;

	direction = velocity.getUnitary();

	_rightSensor = new DistanceSensor(direction.turnRight(), sensorRange, world, this);
	_frontSensor = new DistanceSensor(direction, sensorRange, world, this);
	_leftSensor = new DistanceSensor(direction.turnLeft(), sensorRange, world, this);
	/* Metodo que asigna los valores que corresponden a cada variable definida */
}

void Robot::turnLeft() {
	_velocity = _velocity.turnLeft();
	_leftSensor->turnLeft();
	_frontSensor->turnLeft();
	_rightSensor->turnLeft();
	/* Metodo que implementa el giro a la izquierda del robot con los sensores */
}

void Robot::turnRight() {
	_velocity = _velocity.turnRight();
	_leftSensor->turnRight();
	_frontSensor->turnRight();
	_rightSensor->turnRight();
	/* Metodo que implementa el giro a la derecha del robot on los sensores */
}

void Robot::operator=(Robot robot) {
	_position = robot.getPosition();
	_velocity = robot.getVelocity();
	/* Metodo que actualiza la posicion y velocidad del robot en caso de igualdad de valores */
}

void Robot::moveDeltaT(double deltaT) {
	_position = _position + _velocity * deltaT;
	/* Metodo que implementa el movimiento del robot al interior del mundo */
}

void Robot::moveRobot(double deltaT) {
	if (_frontSensor->senseWall()) this->turnRight();
	else this->moveDeltaT(deltaT);
	/* Metodo que permite el movimiento del robot en el laberinto */
}
