/*
 * Clase principal del programa.
 * Implementa los constructores y metodos esenciales que permiten el inicio del
 * programa, asi como la interaccion entre todas las clases.
 *
 * Autor: Matias Contreras, Francisco Frez, Damian Quiroz.
 * Version: 1.0
 * Since: 2018-07-21
 */

#include <iostream>			// Biblioteca para las operaciones de entrada y salida
#include <fstream>			// Biblioteca para el tratamiendo de archivos
#include <stdlib.h>
#include "MyWorld.h"		// Biblioteca de cabecera que define el mundo a utilizar
#include "Robot.h"			// Biblioteca de cabecera que permite usar robots
#include "Vector2D.h"		// Biblioteca de cabecera que permite usar vectores
#include "Maze.h"			// Biblioteca de cabecera para el uso del laberinto
using namespace std;		// Incluye espacio de nombres para palabras reservadas

int main(int argc, char* argv[]) {
	ifstream maze_in;					// Laberinto original
	ofstream maze_out;					// Laberinto donde se escribe la ruta

	if (argc != 3) {
		cout << "Usage: ./Stage3 <pbm input file> <pbm output file>" << endl;
		exit(1);
	}

	maze_in.open(argv[1]);		// maze_in.open("simple_maze.pbm");
	if (maze_in.fail()) {
		cout << "Error al abrir el archivo " << argv[1] << endl;
		exit(1);
	}

	maze_out.open(argv[2]);		// maze_out.open("robot_route.pbm");
	if (maze_out.fail()) {
		cout << "Error al abrir el archivo " << argv[2] << endl;
		exit(1);
	}

	double deltaT = 1.0; 				// [s]
	double sensorRange = 5; 			// [m]
	MyWorld world;						// crea el mundo simulado
	Robot* robot;						// crea el robot
	Vector2D position(30, 50);			// [m]
	Vector2D velocity(1.0, 0.0);		// [m/s]

	robot = new Robot(position, velocity, sensorRange, &world);		// otorga caracteristicas al robot

	Maze maze(maze_in);							// inicia el laberinto
	Maze route(maze);							// comienza a escribir la ruta
	world.setMaze(maze);						// setea el laberinto en el mundo
	world.setRobot(robot);						// setea el robot en el mundo
	world.printStateDescription();				// imprime en consola el estado
	world.simulate(deltaT, 1000, route);		// inicia simulacion
	route.write(maze_out);						// escribe la ruta en el laberinto de salida

	maze_in.close();			// Cierra maze_in.pbm
	maze_out.close();			// Cierra maze_out.pbm

	return 0;
}
