/*
 * Clase principal del programa.
 * Implementa los constructores y metodos esenciales que permiten el inicio del
 * programa, asi como la interaccion entre todas las clases.
 *
 * Autor: Matias Contreras, Francisco Frez, Damian Quiroz.
 * Version: 2.0
 * Since: 2018-07-29
 */

#include <iostream>				// Biblioteca para las operaciones de entrada y salida
#include <stdlib.h>
#include <fstream>				// Biblioteca para el tratamiendo de archivos
#include <string> 				// Biblioteca para utilizar funciones sobre strings
#include <limits>				// Biblioteca para funciones que definen limites
#include "MyWorld.h"			// Biblioteca de cabecera que define el mundo a utilizar
#include "Robot.h"				// Biblioteca de cabecera que permite usar robots
#include "Vector2D.h"			// Biblioteca de cabecera que permite usar vectores
#include "Maze.h"				// Biblioteca de cabecera para el uso del laberinto
#include "DocumentedMaze.h"		// Biblioteca de cabecera para obtener informaci�n del laberinto
using namespace std;			// Incluye espacio de nombres para palabras reservadas

int main(int argc, char* argv[]) {
	ifstream maze_in;					// Laberinto original
	ofstream maze_out;					// Laberinto donde se escribe la ruta

	if (argc < 3) {
		cout << "Usage: ./Stage4 <pbm input file> <pbm output file> [x  y  dx  dy  fx  fy  r]" << endl;
		exit(1);
	}

	maze_in.open(argv[1]);		// maze_in.open("simple_maze.pbm");
	if (maze_in.fail()) {
		cout << "Error al abrir el archivo " << argv[1] << endl;
		exit(1);
	}

	maze_out.open(argv[2]);		// maze_out.open("robot_route.pbm");
	if (maze_out.fail()) {
		cout << "Error al abrir el archivo " << argv[2] << endl;
		exit(1);
	}

	DocumentedMaze maze(maze_in);
	Maze route(maze);							// comienza a escribir la ruta

	if (argc > 4)
		maze.setEntrance(atof(argv[3]), atof(argv[4]));		// posicion de entrada (x,y)
	else if (argc > 6)
		maze.setDirection(atof(argv[5]), atof(argv[6]));	// direccion de entrada (dx,dy)
	else if (argc > 8)
		maze.setExit(atof(argv[7]), atof(argv[8]));			// salida del laberinto (fx,fy)
	else if (argc > 9)
		maze.setExitRadius(atof(argv[9]));					// radio de salida r

	double deltaT = 1.0; 				// [s]
	double speed = 1.0;					// [m/s]
	double sensorRange = 5; 			// [m]
	MyWorld world;		// crea el mundo simulado
	Robot* robot;						// crea el robot

	Vector2D position = maze.getEntrance();								// vector de posicion inicial
	Vector2D velocity = maze.getDirection().getUnitary() * speed;		// vector de velocidad inicial
	robot = new Robot(position, velocity, sensorRange, &world);			// otorga caracteristicas al robot

	world.setMaze(maze);					// setea el laberinto en el mundo
	world.setRobot(robot);					// setea el robot en el mundo
	world.printStateDescription();			// imprime en consola el estado
	world.simulate(deltaT, route);			// inicia simulacion hasta que salga del laberinto
	route.write(maze_out);					// escribe la ruta en el laberinto de salida

	maze_in.close();			// Cierra maze_in.pbm
	maze_out.close();			// Cierra maze_out.pbm
	return 0;
}
